import ctypes as ct
import os
from contextlib import contextmanager


def _load_library(name, load_directory):
    """Cross-platform dynamic library loader:

    :param name: Name of the library WITHOUT `lib` prefix and WITHOUT extension.
    :param load_directory: Directory containing the library file.
    :returns: Library object.
    """
    name = os.fsdecode(name)
    load_directory = os.fsdecode(load_directory)

    import sys

    prefix = "lib"
    ext = ".so"
    if sys.platform.startswith("darwin"):
        ext = ".dylib"
    elif sys.platform.startswith("win"):
        prefix = ""
        ext = ".dll"
    libpath = os.path.join(load_directory, prefix + name + ext)
    return ct.cdll[libpath]


_libwigxjpf = _load_library("wigxjpf_shared", os.path.dirname(__file__))
_libwigxjpf.wig_table_init.argtypes = [ct.c_int] * 2
_libwigxjpf.wig_table_init.restype = ct.c_void_p
_libwigxjpf.wig_temp_init.argtypes = [ct.c_int]
_libwigxjpf.wig_temp_init.restype = ct.c_void_p
_libwigxjpf.wig_table_free.argtypes = []
_libwigxjpf.wig_table_free.restype = ct.c_void_p
_libwigxjpf.wig_temp_free.argtypes = []
_libwigxjpf.wig_temp_free.restype = ct.c_void_p
_libwigxjpf.wig3jj.argtypes = [ct.c_int] * 6
_libwigxjpf.wig3jj.restype = ct.c_double
_libwigxjpf.wig6jj.argtypes = [ct.c_int] * 6
_libwigxjpf.wig6jj.restype = ct.c_double
_libwigxjpf.wig9jj.argtypes = [ct.c_int] * 9
_libwigxjpf.wig9jj.restype = ct.c_double


def table_init(max_two_j, wigner_type=9):
    return _libwigxjpf.wig_table_init(max_two_j, wigner_type)


def table_free():
    return _libwigxjpf.wig_table_free()


def temp_init(max_two_j):
    return _libwigxjpf.wig_temp_init(max_two_j)


def temp_free():
    return _libwigxjpf.wig_temp_free()


def wig3jj(jj1, jj2, jj3, mm1, mm2, mm3):
    return _libwigxjpf.wig3jj(jj1, jj2, jj3, mm1, mm2, mm3)


def wig6jj(jj1, jj2, jj3, mm1, mm2, mm3):
    return _libwigxjpf.wig6jj(jj1, jj2, jj3, mm1, mm2, mm3)


def wig9jj(jj1, jj2, jj3, jj4, jj5, jj6, jj7, jj8, jj9):
    return _libwigxjpf.wig9jj(jj1, jj2, jj3, jj4, jj5, jj6, jj7, jj8, jj9)


@contextmanager
def wigxjpf(max_two_j, wigner_type=9):
    table_init(max_two_j, wigner_type)
    temp_init(max_two_j)
    try:
        yield None
    finally:
        temp_free()
        table_free()


if __name__ == "__main__":
    with wigxjpf(100):
        val3j = wig3jj(2 * 10, 2 * 15, 2 * 10, 2 * (-3), 2 * 12, 2 * (-9))
        print("3J(10  15  10; -3  12  -9): %.12g" % val3j)

        val6j = wig6jj(2 * 10, 2 * 15, 2 * 10, 2 * 7, 2 * 7, 2 * 9)
        print("6J{10  15  10;  7   7   9}: %.12g" % val6j)

        val9j = wig9jj(1, 2, 3, 4, 6, 8, 3, 6, 9)
        print("9J{0.5 1 1.5; 2 3 4; 1.5 3 4.5}: %.12g" % val9j)
